/*
 *  File:    ca5var.c
 *  Purpose: Communicating from Tcl to C. Needed to set parameters before
 *           starting the Prolog.
 */

#include <string.h>
#include "a5.h"
#include "ca5var.h"

static int var[A5_END];

static int name_to_index(char *name) 
{
  int i;
  for(i = 0; i < A5_END; i++) {
    if(strcmp(name, A5_PRINTNAME(i)) == 0) return i;
  }
  return -1;
}

/* Implements the Tcl command "a5_var_clear_all" */
int ET_COMMAND_a5_var_clear_all(ET_TCLARGS)
{
  int i;
  if(argc != 1) {
    Et_ResultF(interp,"wrong # args: should be \"%s\"", argv[0]);
    return TCL_ERROR;
  }
  for(i = 0; i < A5_END; i++) 
    var[i] = 0;
  return TCL_OK;
}

/* Implements the Tcl command "a5_var_set INDEX VALUE" */
int ET_COMMAND_a5_var_set(ET_TCLARGS)
{
  int index, value;
  if(argc != 3) {
    Et_ResultF(interp,"wrong # args: should be \"%s INDEX VALUE\"", argv[0]);
    return TCL_ERROR;
  }
  index = name_to_index(argv[1]);
  if(index < 0 || index >= A5_END) {
    Et_ResultF(interp,"illegal index of A5 variable in \"%s %s %s\"", 
	       argv[0], argv[1], argv[2]);
    return TCL_ERROR;
  }
  value = atoi(argv[2]);
  var[index] = value;
  return TCL_OK;
}

int a5_sys_var_get(enum a5_vars index)
{
  if(index < 0 || index > A5_END) return -4711;
  else return var[index];
}
