#include "gprolog.h"
#include "a5.h"
#include "gc_c.h"
#include "ca5var.h"

void a5_Init()
{
  int func, res;
  WamWord arg[1];

  func = Find_Atom("start");
  res = Pl_Query_Start(func, 0, arg, TRUE);
  if(res == PL_SUCCESS) {
    Pl_Query_End(PL_RECOVER);
  }
  else {
    /* Maybe some changes in error management here? */
    fprintf(stderr, "Initializing of Prolog engine failed.\n");
    exit(1);
  }
}

void a5_MainLoop()
{
  int func, res;
  WamWord arg[1];

  func = Find_Atom("top");
  res = Pl_Query_Start(func, 0, arg, TRUE); 
  while (Tk_GetNumMainWindows() > 0) {
    Tcl_DoOneEvent(TCL_DONT_WAIT);
    if(prolog_event_pending()) {
      /* fprintf(stderr, "Start PL\n"); */
      if(res == PL_SUCCESS) {
	res = Pl_Query_Next_Solution();
	/* fprintf(stderr, "Done PL\n"); */
      }
      else {
	Pl_Query_End(PL_RECOVER);
	if(GC_Is_Out_Of_Memory()) {
	  fprintf(stderr, "Prolog engine out of memory. Current string\n%s\n",
		  peek_current_string());
	  GC_Init(-1,-1);
	}
	else {
	  /* Maybe some changes in error management here? */
	  fprintf(stderr, "Prolog engine failed. Current string\n%s\n",
		  peek_current_string());
	}
      }
    }
  }
}

int Et_CustomMainLoop(Tcl_Interp *interp)
{
  int debug, stack, argc;
  char *argv[1];
  argv[0] = "AAAAA";
  argc = 1;
  /*  fprintf(stderr, "Starting the engine\n"); */
  debug = a5_sys_var_get(A5_DEBUG);
  stack = a5_sys_var_get(A5_STACK);
  if(stack <= 0) stack = 8192;
  if(stack > 0) {
    def_global_size = stack;
    /*    fprintf(stderr, "Setting global stack to %d KB\n", stack); */
  }
  Start_Prolog(argc, argv);
  /*  fprintf(stderr, "Engine up\n"); */
  GC_Init(-1,-1);
  /*  fprintf(stderr, "GC Initalized\n");*/
  if(debug >= 1) {
    int func, res;
    WamWord arg[1];

    /*    fprintf(stderr, "Debug mode on\n");*/
    func = Find_Atom("set_debug_flag");
    res = Pl_Query_Start(func, 0, arg, TRUE);
    if(res == PL_SUCCESS) {
      Pl_Query_End(PL_RECOVER);
    }
    else {
      /* Maybe some changes in error management here? */
      fprintf(stderr, "Warning: failed to set engine debug flag.\n");
    }
  }
  /*  fprintf(stderr, "Starting a5_Init\n"); */
  a5_Init();
  /*  fprintf(stderr, "Done a5_Init\n");*/
  a5_MainLoop();
  /*  fprintf(stderr, "Done main loop\n");*/
  return TCL_OK;
}

int main(int argc, char *argv[])
{
  Et_Init(argc, argv);
  return 0;
}
 
